(*  GREATIS PRINTER INFORMATION                             *)
(*  Copyright (C) 2005-2014 Greatis Software                *)
(*                                                          *)
(*  http://www.greatis.com/delphicb/printerinfo/            *)

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PrinterInformation, StdCtrls, ShellAPI, ExtCtrls, TypInfo;

type
  TFormMain = class(TForm)
    PrinterInformation: TPrinterInformation;
    MemoPrinterInformation: TMemo;
    Panel1: TPanel;
    ButtonUpdate: TButton;
    Panel2: TPanel;
    ButtonMore: TButton;
    ButtonClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ButtonUpdateClick(Sender: TObject);
    procedure ButtonMoreClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateInformation;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

procedure TFormMain.UpdateInformation;

  function PrinterFlagsToString(Value: TPrinterFlags): string;
  begin
    Result:='';
    if peExpand in Value then Result:=Result+'peExpand ';
    if peContainer in Value then Result:=Result+'peContainer ';
    if peIconTopLevelNetworkName in Value then Result:=Result+'peIconTopLevelNetworkName, ';
    if peIconNetworkDomain in Value then Result:=Result+'peIconNetworkDomain ';
    if peIconPrintServer in Value then Result:=Result+'peIconPrintServer ';
    if peIconPrinter in Value then Result:=Result+'peIconPrinter ';
  end;

  function PrinterAttributesToString(Value: TPrinterAttributes): string;
  begin
    Result:='';
    if paDirect in Value then Result:=Result+'paDirect ';
    if paDoCompleteFirst in Value then Result:=Result+'paDoCompleteFirst ';
    if paEnableDevQueryPrint in Value then Result:=Result+'paEnableDevQueryPrint ';
    if paKeepPrintedJobs in Value then Result:=Result+'paKeepPrintedJobs ';
    if paLocal in Value then Result:=Result+'paLocal ';
    if paNetwork in Value then Result:=Result+'paNetwork ';
    if paPublished in Value then Result:=Result+'paPublished ';
    if paQueued in Value then Result:=Result+'paQueued ';
    if paRawOnly in Value then Result:=Result+'paRawOnly ';
    if paShared in Value then Result:=Result+'paShared ';
    if paFax in Value then Result:=Result+'paFax ';
    if paFriendlyName in Value then Result:=Result+'paFriendlyName ';
    if paMachine in Value then Result:=Result+'paMachine ';
    if paPushedUser in Value then Result:=Result+'paPushedUser ';
    if paPushedMachine in Value then Result:=Result+'paPushedMachine ';
    if paTerminalServer in Value then Result:=Result+'paTerminalServer ';
  end;

  function PaperOrientationToString(Value: TPaperOrientation): string;
  begin
    case Value of
      poPortrait: Result:='Portrait';
      poLandscape: Result:='Landscape';
      else Result:='Unknown';
    end;
  end;

  function DataFieldsToString(Value: TDataFields): string;
  begin
    Result:='';
    if dfOrientation in Value then Result:=Result+'dfOrientation ';
    if dfPaperSize in Value then Result:=Result+'dfPaperSize ';
    if dfPaperLength in Value then Result:=Result+'dfPaperLength ';
    if dfPaperWidth in Value then Result:=Result+'dfPaperWidth ';
    if dfScale in Value then Result:=Result+'dfScale ';
    if dfCopies in Value then Result:=Result+'dfCopies ';
    if dfDefaultSource in Value then Result:=Result+'dfDefaultSource ';
    if dfPrintQuality in Value then Result:=Result+'dfPrintQuality ';
    if dfColor in Value then Result:=Result+'dfColor ';
    if dfDuplex in Value then Result:=Result+'dfDuplex ';
    if dfYResolution in Value then Result:=Result+'dfYResolution ';
    if dfTrueType in Value then Result:=Result+'dfTrueType ';
    if dfCollate in Value then Result:=Result+'dfCollate ';
    if dfFormName in Value then Result:=Result+'dfFormName ';
    if dfICMMethod in Value then Result:=Result+'dfICMMethod ';
    if dfICMIntent in Value then Result:=Result+'dfICMIntent ';
    if dfMediaType in Value then Result:=Result+'dfMediaType ';
    if dfDitherType in Value then Result:=Result+'dfDitherType ';
  end;

  function PaperSourceToString(Value: TPaperSource): string;
  begin
    case Value of
      psAuto: Result:='psAuto';
      psCassette: Result:='psCassette';
      psEnvelope: Result:='psEnvelope';
      psEnvelopeManual: Result:='psEnvelopeManual';
      psFirst: Result:='psFirst';
      psLargeCapacity: Result:='psLargeCapacity';
      psLargeFormat: Result:='psLargeFormat';
      psLast: Result:='psLast';
      psLower: Result:='psLower';
      psManual: Result:='psManual';
      psMiddle: Result:='psMiddle';
      psTractor: Result:='psTractor';
      psSmallFormat: Result:='psSmallFormat';
    else Result:='psUser';
    end;
  end;

  function PrintQualityToString(Value: TPrintQuality): string;
  begin
    case Value of
      pqHigh: Result:='pqHigh';
      pqMedium: Result:='pqMedium';
      pqLow: Result:='pqLow';
      pqDraft: Result:='pqDraft';
    else Result:='pqDPI';
    end;
  end;

  function PrintColorToString(Value: TPrintColor): string;
  begin
    if Value=pcColor then Result:='pcColor'
    else Result:='pcMonochrome';
  end;

  function PrintDuplexToString(Value: TPrintDuplex): string;
  begin
    case Value of
      pdHorizontal: Result:='pdHorizontal';
      pdVertical: Result:='pdVertical';
    else Result:='pdSimplex';
    end;
  end;

  function PrintTrueTypeToString(Value: TPrintTrueType): string;
  begin
    case Value of
      ttBitmap: Result:='ttBitmap';
      ttDownload: Result:='ttDownload';
      ttDownloadOutline: Result:='ttDownloadOutline';
    else Result:='ttSubDev';
    end;
  end;

  function BooleanToString(Value: Boolean): string;
  begin
    if Value then Result:='Yes'
    else Result:='No';
  end;

  function PrintICMMethodToString(Value: TPrintICMMethod): string;
  begin
    case Value of
      imHandledByWindows: Result:='imHandledByWindows';
      imHandledByDriver: Result:='imHandledByDriver';
      imHandledByDevice: Result:='imHandledByDevice';
    else Result:='imDisabled';
    end;
  end;

  function PrintICMIntentToString(Value: TPrintICMIntent): string;
  begin
    case Value of
      iiABSColorimetric: Result:='iiABSColorimetric';
      iiColorimetric: Result:='iiColorimetric';
      iiContrast: Result:='iiContrast';
      iiSaturate: Result:='iiSaturate';
    else Result:='iiUser';
    end;
  end;

  function PrintMediaTypeToString(Value: TPrintMediaType): string;
  begin
    case Value of
      mtGlossyPaper: Result:='mtGlossyPaper';
      mtTransparentFilm: Result:='mtTransparentFilm';
    else Result:='mtPlainPaper';
    end;
  end;

  function PrintDitherTypeToString(Value: TPrintDitherType): string;
  begin
    case Value of
      dtCoarseBrush: Result:='dtCoarseBrush';
      dtFineBrush: Result:='dtFineBrush';
      dtLineArt: Result:='dtLineArt';
      dtGrayScale: Result:='dtGrayScale';
    else Result:='dtNone';
    end;
  end;

begin
  with PrinterInformation,MemoPrinterInformation.Lines do
  begin
    BeginUpdate;
    try
      Clear;
      Add('Content name'#9+ContentName);
      Add('Content description'#9+ContentDescription);
      Add('Content comment'#9+ContentComment);
      Add('Printer flags'#9+PrinterFlagsToString(PrinterFlags));
      Add('Server name'#9+ServerName);
      Add('Printer name'#9+PrinterName);
      Add('Share name'#9+ShareName);
      Add('Port name'#9+PortName);
      Add('Driver name'#9+DriverName);
      Add('Comment'#9+Comment);
      Add('Location'#9+Location);
      Add('Page separator file'#9+SepFile);
      Add('Print processor'#9+PrintProcessor);
      Add('Data type'#9+DataType);
      Add('Parameters'#9+Parameters);
      Add('Security descriptor'#9+IntToHex(Integer(SecurityDescriptor),8));
      Add('Attributes'#9+PrinterAttributesToString(Attributes));
      Add('Priority'#9+IntToStr(Priority));
      Add('Default priority'#9+IntToStr(DefaultPriority));
      Add('Start time'#9+IntToStr(StartTime));
      Add('Until time'#9+IntToStr(UntilTime));
      Add('Job count'#9+IntToStr(JobCount));
      Add('Average page per minute'#9+IntToStr(AveragePPM));
      Add('Device name'#9+DeviceName);
      Add('Specification version'#9+SpecVersionString);
      Add('Driver version'#9+DriverVersionString);
      Add('Data fields'#9+DataFieldsToString(DataFields));
      if dfOrientation in DataFields then Add('Paper orientation'#9+PaperOrientationToString(PaperOrientation));
      if dfPaperSize in DataFields then
      begin
        Add('Paper size'#9+IntToStr(PaperSize));
        Add('Paper size name'#9+PaperSizeName);
      end;
      if dfPaperLength in DataFields then Add('Paper length'#9+IntToStr(PaperLength));
      if dfPaperWidth in DataFields then Add('Paper width'#9+IntToStr(PaperWidth));
      if dfScale in DataFields then Add('Scale'#9+IntToStr(Scale)+'/100');
      if dfCopies in DataFields then Add('Copies'#9+IntToStr(Copies));
      if dfDefaultSource in DataFields then Add('Default source'#9+PaperSourceToString(DefaultSource));
      if dfPrintQuality in DataFields then Add('Print quality'#9+PrintQualityToString(PrintQuality));
      if dfXResolution in DataFields then Add('X resolution'#9+IntToStr(XResolution));
      if dfYResolution in DataFields then Add('Y resolution'#9+IntToStr(YResolution));
      if dfColor in DataFields then Add('Color'#9+PrintColorToString(Color));
      if dfDuplex in DataFields then Add('Duplex'#9+PrintDuplexToString(Duplex));
      if dfTrueType in DataFields then Add('TrueType'#9+PrintTrueTypeToString(TrueType));
      if dfCollate in DataFields then Add('Collate'#9+BooleanToString(Collate));
      if dfFormName in DataFields then Add('Form name'#9+FormName);
      if dfICMMethod in DataFields then Add('ICM method'#9+PrintICMMethodToString(ICMMethod));
      if dfICMIntent in DataFields then Add('ICM Intent'#9+PrintICMIntentToString(ICMIntent));
      if dfMediaType in DataFields then Add('Media type'#9+PrintMediaTypeToString(MediaType));
      if dfDitherType in DataFields then Add('Dither type'#9+PrintDitherTypeToString(DitherType));
    finally
      EndUpdate;
    end;
  end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
  LF: TLogFont;
begin
  SystemParametersInfo(SPI_GETICONTITLELOGFONT,SizeOf(LF),@LF,0);
  with LF,Font do
  begin
    ScaleBy(Abs(lfHeight),Abs(Height));
    Handle:=CreateFontIndirect(LF);
  end;
end;

procedure TFormMain.FormShow(Sender: TObject);
var
  TabSize: DWORD;
begin
  TabSize:=120;
  SendMessage(MemoPrinterInformation.Handle,EM_SETTABSTOPS,1,Integer(@TabSize));
  UpdateInformation;
end;

procedure TFormMain.ButtonUpdateClick(Sender: TObject);
begin
  PrinterInformation.Update;
  UpdateInformation;
end;

procedure TFormMain.ButtonMoreClick(Sender: TObject);
begin
  ShellExecute(Handle,'open','http://www.greatis.com/delphicb/printerinfo/',nil,nil,SW_SHOWDEFAULT);
end;

procedure TFormMain.ButtonCloseClick(Sender: TObject);
begin
  Application.Terminate;
end;

end.
